/*	$Id$ */

/*
 * Copyright (c) 1999,2008 LAAS/CNRS                       --  Mon Aug  7 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef _SCORRELSTRUCT_H
#define _SCORRELSTRUCT_H

#include "stereopixelClient.h"

#define SPIX_PATH_MAX	256

typedef struct SpixName {
  char id[SPIX_PATH_MAX];
} SpixName;


/*
 * Correlation control --------------------------------------------------
 */

typedef enum SpixCorrelType {

   SPIXCT_ZNCC,
   SPIXCT_ZNCC_SIGMA,
   SPIXCT_CENSUS,
   SPIXCT_CENSUS_SIGMA

#define SPIXDEF_CORRELTYPE	SPIXCT_ZNCC

} SpixCorrelType;

typedef enum SpixInterpolationType {

   SPIXIT_NO_INTERPOLATION,
   SPIXIT_PARABOLIC_INTERPOLATION,
   SPIXIT_BILINEAR_INTERPOLATION,
   SPIXIT_BOTH_INTERPOLATIONS

#define SPIXDEF_INTERPOLATION	SPIXIT_PARABOLIC_INTERPOLATION

} SpixInterpolationType;

typedef struct SpixCorrelParams {

   int win_nb_lig, win_nb_col;	/* Taille de la fentre de corrlation */
   int disp_min, disp_max;	/* cart de disparit */

   SpixCorrelType correlType; /* Indique si on fait ZNCC (0) ou CENSUS (2), et
			     * si on dtermine les carts types ou non (1)
			     * ou (3). */

   SpixInterpolationType interpolationType;
				/* Indique le type d'interpolation effectu sur
				 * la disparit */

   /*
    * 'th' stands for 'threshold'. Les diffrents seuils pour valider un
    * apariement. Dans l'ordre : seuil sur l'cart type de la fentre
    * considre, sur la valeur du spixore max, sur l'cart entre les 2 plus
    * grand pics, sur la diffrence max / voisins, et sur l'cart (en
    * pixels) corrlation directe / corrlation inverse.
    */

   double var_th;
   double score_th;
   double diff_neighbor_max_th;
   int inv_th;
   int threads;

#define SPIXDEF_WINLIG	9
#define SPIXDEF_WINCOL	9
#define SPIXDEF_DISPMIN	-240
#define SPIXDEF_DISPMAX	0
#define SPIXDEF_VARTH	1.0
#define SPIXDEF_SCORETH	0.0
#define SPIXDEF_DIFF2NEIGH	0.0
#define SPIXDEF_INVTH	1
#define SPIXDEF_THREADS	1

} SpixCorrelParams;


/*
 * Blob filter ----------------------------------------------------------
 */

typedef struct SpixBlobFilter {

   int enableBlobFilter;
   int sizeMin;
   float diffMax;

#define SPIXDEF_BLOB		1
#define SPIXDEF_BLOBSIZEMIN	500
#define SPIXDEF_BLOBDIFFMAX	1.0

} SpixBlobFilter;


/*
 * Reconstruction -------------------------------------------------------
 */

typedef enum SpixFrameCoord {

   SPIX_RECTIFIED_FRAME,
   SPIX_LEFT_CAMERA_FRAME

#define SPIXDEF_FRAME	SPIX_RECTIFIED_FRAME

} SpixFrameCoord;


/*
 * File ops -------------------------------------------------------------
 */

typedef struct SpixSaveParams {

  char baseName[SPIX_PATH_MAX];
  int numDisp;
  int numDispCV;
  int num3D;
  int unused;

#define SPIXDEF_FRAMEBASENAME	"."

} SpixSaveParams;

typedef struct SpixCorrelationResult {
  int nbPoints;

  int nbRemovedByUnicity;	/* The number of matches for which the unicity
				   constraint has not been satisfied */
  int nbRemovedByBlob;		/* The number of matches that have been
				   discarded by the disparity blob filter */
  int nbCorrelated;		/* The number of pixels that have been
				   correlated */
} SpixCorrelationResult;

#endif /* _SCORRELSTRUCT_H */
